@@ -9,6 +9,7 @@
 #include "FlatpakBackend.h"
 #include "FlatpakFetchDataJob.h"
 #include "FlatpakSourcesBackend.h"
+#include "LazyIconResolver.h"
 #include "config-paths.h"
 #include "libdiscover_backend_flatpak_debug.h"
 
@@ -25,7 +26,6 @@
 #include <KDesktopFile>
 #include <KFormat>
 #include <KIO/ApplicationLauncherJob>
-#include <KIconLoader>
 #include <KLocalizedString>
 
 #include <AppStreamQt/release.h>
@@ -33,6 +33,7 @@
 #include <QDesktopServices>
 #include <QDir>
 #include <QDirIterator>
+#include <QEvent>
 #include <QFileInfo>
 #include <QFutureWatcher>
 #include <QIcon>
@@ -40,6 +41,7 @@
 #include <QNetworkReply>
 #include <QNetworkRequest>
 #include <QProcess>
+#include <QQueue>
 #include <QStringList>
 #include <QTemporaryFile>
 #include <QTimer>
@@ -79,33 +81,40 @@ FlatpakResource::FlatpakResource(const AppStream::Component &component, FlatpakI
     // Start fetching remote icons during initialization
     const auto icons = m_appdata.icons();
     m_stockIcon = std::ranges::any_of(icons, [](const AppStream::Icon &icon) {
-        return icon.kind() == AppStream::Icon::KindStock && KIconLoader::global()->hasIcon(icon.name());
+        return icon.kind() == AppStream::Icon::KindStock && AppStreamUtils::kIconLoaderHasIcon(icon.name());
+    });
+    connect(this, &AbstractResource::iconChanged, this, [this] {
+        Q_EMIT backend()->resourcesChanged(this, {"icon"});
     });
     if (!m_stockIcon && !icons.isEmpty() && !std::ranges::any_of(icons, [](const AppStream::Icon &icon) {
             return icon.kind() == AppStream::Icon::KindLocal || icon.kind() == AppStream::Icon::KindCached;
         })) {
-        const auto icon = icons.constFirst();
-        const QString fileName = iconCachePath(icon);
-        if (!QFileInfo::exists(fileName)) {
-            const QDir cacheDir(QStandardPaths::writableLocation(QStandardPaths::CacheLocation));
-            // Create $HOME/.cache/discover/icons folder
-            cacheDir.mkdir(QStringLiteral("icons"));
-            auto reply = manager->get(QNetworkRequest(icon.url()));
-            connect(reply, &QNetworkReply::finished, this, [this, icon, fileName, reply] {
-                if (reply->error() == QNetworkReply::NoError) {
-                    QByteArray iconData = reply->readAll();
-                    QFile file(fileName);
-                    if (file.open(QIODevice::WriteOnly)) {
-                        file.write(iconData);
-                    } else {
-                        qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "could not find icon for" << packageName() << reply->url();
-                        QIcon::fromTheme(QStringLiteral("package-x-generic")).pixmap(32, 32).toImage().save(fileName);
+        for (const auto &icon : icons) {
+            if (icon.kind() != AppStream::Icon::KindRemote) {
+                continue;
+            }
+            const QString fileName = iconCachePath(icon);
+            if (!QFileInfo::exists(fileName)) {
+                const QDir cacheDir(QStandardPaths::writableLocation(QStandardPaths::CacheLocation));
+                // Create $HOME/.cache/discover/icons folder
+                cacheDir.mkdir(QStringLiteral("icons"));
+                auto reply = manager->get(QNetworkRequest(icon.url()));
+                connect(reply, &QNetworkReply::finished, this, [this, icon, fileName, reply] {
+                    if (reply->error() == QNetworkReply::NoError) {
+                        QByteArray iconData = reply->readAll();
+                        QFile file(fileName);
+                        if (file.open(QIODevice::WriteOnly)) {
+                            file.write(iconData);
+                        } else {
+                            qCDebug(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "could not find icon for" << packageName() << reply->url();
+                            QIcon::fromTheme(QStringLiteral("package-x-generic")).pixmap(32, 32).toImage().save(fileName);
+                        }
+                        file.close();
+                        Q_EMIT iconChanged();
+                        reply->deleteLater();
                     }
-                    file.close();
-                    Q_EMIT iconChanged();
-                    reply->deleteLater();
-                }
-            });
+                });
+            }
         }
     }
 
@@ -132,11 +141,7 @@ QList<PackageState> FlatpakResource::addonsInformation()
 QString FlatpakResource::availableVersion() const
 {
     if (m_availableVersion.isEmpty()) {
-#if ASQ_CHECK_VERSION(1, 0, 0)
         const auto releases = m_appdata.releasesPlain().entries();
-#else
-        const auto releases = m_appdata.releases();
-#endif
         if (!releases.isEmpty()) {
             auto latestVersion = releases.constFirst().version();
             for (const auto &release : releases) {
@@ -194,12 +199,11 @@ void FlatpakResource::updateFromAppStream()
     updateFromRef(ref);
 }
 
-QStringList FlatpakResource::categories()
+bool FlatpakResource::hasCategory(const QString &category) const
 {
-    auto cats = m_appdata.categories();
-    if (m_appdata.kind() != AppStream::Component::KindAddon)
-        cats.append(QStringLiteral("Application"));
-    return cats;
+    if (m_appdata.kind() != AppStream::Component::KindAddon && category == QStringLiteral("Application"))
+        return true;
+    return m_appdata.hasCategory(category);
 }
 
 QString FlatpakResource::comment()
@@ -217,16 +221,16 @@ quint64 FlatpakResource::downloadSize() const
     return m_downloadSize;
 }
 
-QVariant FlatpakResource::icon() const
+void FlatpakResource::resolveIcon()
 {
-    QIcon ret;
+    m_icon = QIcon();
     const auto icons = m_appdata.icons();
 
     if (!m_bundledIcon.isNull()) {
-        ret = QIcon(m_bundledIcon);
+        m_icon = QIcon(m_bundledIcon);
     } else if (icons.isEmpty()) {
-        ret = QIcon::fromTheme(QStringLiteral("package-x-generic"));
-    } else
+        m_icon = QIcon::fromTheme(QStringLiteral("package-x-generic"));
+    } else {
         for (const AppStream::Icon &icon : icons) {
             switch (icon.kind()) {
             case AppStream::Icon::KindLocal:
@@ -239,23 +243,24 @@ QVariant FlatpakResource::icon() const
                     while (dit.hasNext()) {
                         const auto currentPath = dit.next();
                         if (dit.fileName() == path) {
-                            ret.addFile(currentPath, icon.size());
+                            m_icon->addFile(currentPath, icon.size());
                         }
                     }
                 } else {
-                    ret.addFile(path, icon.size());
+                    m_icon->addFile(path, icon.size());
                 }
-            } break;
+                break;
+            }
             case AppStream::Icon::KindStock: {
                 if (m_stockIcon) {
-                    return QIcon::fromTheme(icon.name());
+                    m_icon = QIcon::fromTheme(icon.name());
                 }
                 break;
             }
             case AppStream::Icon::KindRemote: {
                 const QString fileName = iconCachePath(icon);
                 if (QFileInfo::exists(fileName)) {
-                    ret.addFile(fileName, icon.size());
+                    m_icon->addFile(fileName, icon.size());
                 }
                 break;
             }
@@ -263,12 +268,23 @@ QVariant FlatpakResource::icon() const
                 break;
             }
         }
-
-    if (ret.isNull()) {
-        ret = QIcon::fromTheme(QStringLiteral("package-x-generic"));
     }
 
-    return ret;
+    if (m_icon->isNull()) {
+        m_icon = QIcon::fromTheme(QStringLiteral("package-x-generic"));
+    }
+
+    Q_EMIT iconChanged();
+}
+
+QVariant FlatpakResource::icon() const
+{
+    if (m_icon.has_value()) {
+        return m_icon.value();
+    }
+
+    LazyIconResolver::instance()->queue(const_cast<FlatpakResource *>(this));
+    return u"package-x-generic"_s;
 }
 
 QString FlatpakResource::installedVersion() const
@@ -293,7 +309,7 @@ AbstractResource::Type FlatpakResource::type() const
 {
     switch (m_type) {
     case FlatpakResource::Runtime:
-        return Technical;
+        return ApplicationSupport;
     case FlatpakResource::Extension:
         return Addon;
     default:
@@ -518,7 +534,14 @@ void FlatpakResource::invokeApplication() const
 
 void FlatpakResource::fetchChangelog()
 {
-    Q_EMIT changelogFetched(AppStreamUtils::changelogToHtml(m_appdata));
+    if (m_permissions.isEmpty()) {
+        loadPermissions();
+    }
+    QString changelog = AppStreamUtils::changelogToHtml(m_appdata);
+    if (!m_newPermissions.isEmpty()) {
+        changelog += "<h3>New Permissions</h3>\n<ul>"_L1 + m_newPermissions + "</ul>"_L1;
+    }
+    Q_EMIT changelogFetched(changelog);
 }
 
 void FlatpakResource::fetchScreenshots()
@@ -541,13 +564,26 @@ void FlatpakResource::setBundledIcon(const QPixmap &pixmap)
     m_bundledIcon = pixmap;
 }
 
+void FlatpakResource::reportSizeChange()
+{
+    Q_EMIT sizeChanged();
+
+    if (!qobject_cast<FlatpakBackend *>(backend())->isFetching()) {
+        static const QVector<QByteArray> properties = {
+            "size",
+            "sizeDescription",
+        };
+        Q_EMIT backend()->resourcesChanged(this, properties);
+    }
+}
+
 void FlatpakResource::setDownloadSize(quint64 size)
 {
     m_downloadSize = size;
 
     setPropertyState(DownloadSize, AlreadyKnown);
 
-    Q_EMIT sizeChanged();
+    reportSizeChange();
 }
 
 void FlatpakResource::setFlatpakFileType(FlatpakFileType fileType)
@@ -571,7 +607,7 @@ void FlatpakResource::setInstalledSize(quint64 size)
 
     setPropertyState(InstalledSize, AlreadyKnown);
 
-    Q_EMIT sizeChanged();
+    reportSizeChange();
 }
 
 void FlatpakResource::setOrigin(const QString &origin)
@@ -653,14 +689,11 @@ QUrl FlatpakResource::url() const
 
 QDate FlatpakResource::releaseDate() const
 {
-#if ASQ_CHECK_VERSION(1, 0, 0)
-    if (!m_appdata.releasesPlain().isEmpty()) {
-        auto release = m_appdata.releasesPlain().indexSafe(0).value();
-#else
-    if (const auto releases = m_appdata.releases(); !releases.isEmpty()) {
-        auto release = releases.constFirst();
-#endif
-        return release.timestamp().date();
+    if (const auto releases = m_appdata.releasesPlain(); !releases.isEmpty()) {
+        auto release = releases.indexSafe(0);
+        if (release) {
+            return release->timestamp().date();
+        }
     }
 
     return {};
@@ -721,19 +754,13 @@ QString FlatpakResource::versionString()
         if (ref) {
             version = QString::fromUtf8(flatpak_installed_ref_get_appdata_version(ref));
         }
-#if ASQ_CHECK_VERSION(1, 0, 0)
     } else if (!m_appdata.releasesPlain().isEmpty()) {
         const auto release = m_appdata.releasesPlain().indexSafe(0).value();
-#else
-    } else if (!m_appdata.releases().isEmpty()) {
-        const auto release = m_appdata.releases().constFirst();
-#endif
         version = release.version();
     } else {
         version = m_id.branch;
     }
-
-    return AppStreamUtils::versionString(version, m_appdata);
+    return version;
 }
 
 QString translateSymbolicName(const QStringView &name)
@@ -799,9 +826,18 @@ QString createHtmlList(const QStringList &itemList)
     return str;
 }
 
+bool FlatpakResource::updateNeedsAttention()
+{
+    if (m_permissions.isEmpty()) {
+        loadPermissions();
+    }
+    return !m_newPermissions.isEmpty();
+}
+
 void FlatpakResource::loadPermissions()
 {
-    QByteArray metaDataBytes = FlatpakRunnables::fetchMetadata(this, nullptr);
+    auto b = qobject_cast<FlatpakBackend *>(backend());
+    QByteArray metaDataBytes = FlatpakRunnables::fetchMetadata(this, b->cancellable());
 
     QTemporaryFile f;
     if (!f.open()) {
@@ -809,8 +845,20 @@ void FlatpakResource::loadPermissions()
     }
     f.write(metaDataBytes);
     f.close();
+    m_permissions = readPermissions(f.fileName());
 
-    KDesktopFile parser(f.fileName());
+    auto currentPermissions = readPermissions(installPath() + "/metadata"_L1);
+    for (const auto &permission : std::as_const(m_permissions)) {
+        if (!currentPermissions.contains(permission)) {
+            m_newPermissions += "<li>"_L1 + i18n("%1: %2", permission.brief(), permission.description()) + "</li>\n"_L1;
+        }
+    }
+}
+
+QVector<FlatpakPermission> FlatpakResource::readPermissions(const QString &file) const
+{
+    QVector<FlatpakPermission> permissions;
+    KDesktopFile parser(file);
 
     QString brief, description;
 
@@ -822,7 +870,7 @@ void FlatpakResource::loadPermissions()
     if (shared.contains("network"_L1)) {
         brief = i18n("Network Access");
         description = i18n("Can access the internet");
-        m_permissions.append(FlatpakPermission(brief, description, u"network-wireless-symbolic"_s));
+        permissions.append(FlatpakPermission(brief, description, u"network-wireless-symbolic"_s));
     }
 
     const QString sockets = contextGroup.readEntry("sockets", QString());
@@ -831,64 +879,64 @@ void FlatpakResource::loadPermissions()
 
     if (sockets.contains("pulseaudio"_L1) || filesystems.contains("xdg-run/pipewire-0"_L1)) {
         brief = i18n("Sound system access");
-        description = i18n("Can play audio");
-        m_permissions.append(FlatpakPermission(brief, description, u"audio-speakers-symbolic"_s));
+        description = i18n("Can play and record audio");
+        permissions.append(FlatpakPermission(brief, description, u"audio-speakers-symbolic"_s));
     }
     if (sockets.contains("session-bus"_L1)) {
         brief = i18n("Session Bus Access");
         description = i18n("Can communicate with all other applications and processes run in this user account");
-        m_permissions.append(FlatpakPermission(brief, description, u"security-medium-symbolic"_s));
+        permissions.append(FlatpakPermission(brief, description, u"security-medium-symbolic"_s));
         fullSessionBusAccess = true;
     }
     if (sockets.contains("system-bus"_L1)) {
         brief = i18n("System Bus Access");
         description = i18n("Can communicate with all other applications and processes on the system");
-        m_permissions.append(FlatpakPermission(brief, description, u"security-medium-symbolic"_s));
+        permissions.append(FlatpakPermission(brief, description, u"security-medium-symbolic"_s));
         fullSystemBusAccess = true;
     }
     if (sockets.contains("ssh-auth"_L1)) {
         brief = i18n("Remote Login Access");
         description = i18n("Can initiate remote login requests using the SSH protocol");
-        m_permissions.append(FlatpakPermission(brief, description, u"x-shape-connection-symbolic"_s));
+        permissions.append(FlatpakPermission(brief, description, u"x-shape-connection-symbolic"_s));
     }
     if (sockets.contains("pcsc"_L1)) {
         brief = i18n("Smart Card Access");
         description = i18n("Can integrate and communicate with smart cards");
-        m_permissions.append(FlatpakPermission(brief, description, u"auth-sim-symbolic"_s));
+        permissions.append(FlatpakPermission(brief, description, u"auth-sim-symbolic"_s));
     }
     if (sockets.contains("cups"_L1)) {
         brief = i18n("Printer Access");
         description = i18n("Can integrate and communicate with printers");
-        m_permissions.append(FlatpakPermission(brief, description, u"printer-symbolic"_s));
+        permissions.append(FlatpakPermission(brief, description, u"printer-symbolic"_s));
     }
     if (sockets.contains("gpg-agent"_L1)) {
         brief = i18n("GPG Agent");
         description = i18n("Allows access to the GPG cryptography service, generally used for signing and reading signed documents");
-        m_permissions.append(FlatpakPermission(brief, description, u"document-edit-sign-encrypt"_s));
+        permissions.append(FlatpakPermission(brief, description, u"document-edit-sign-encrypt"_s));
     }
 
     const QString features = contextGroup.readEntry("features", QString());
     if (features.contains("bluetooth"_L1)) {
         brief = i18n("Bluetooth Access");
         description = i18n("Can integrate and communicate with Bluetooth devices");
-        m_permissions.append(FlatpakPermission(brief, description, u"network-bluetooth-symbolic"_s));
+        permissions.append(FlatpakPermission(brief, description, u"network-bluetooth-symbolic"_s));
     }
     if (features.contains("devel"_L1)) {
         brief = i18n("Low-Level System Access");
         description = i18n("Can make low-level system calls (e.g. ptrace)");
-        m_permissions.append(FlatpakPermission(brief, description, u"run-build-symbolic"_s));
+        permissions.append(FlatpakPermission(brief, description, u"run-build-symbolic"_s));
     }
 
     const QString devices = contextGroup.readEntry("devices", QString());
     if (devices.contains("all"_L1)) {
         brief = i18n("Device Access");
         description = i18n("Can communicate with and control built-in or connected hardware devices");
-        m_permissions.append(FlatpakPermission(brief, description, u"device-notifier-symbolic"_s));
+        permissions.append(FlatpakPermission(brief, description, u"device-notifier-symbolic"_s));
     }
     if (devices.contains("kvm"_L1)) {
         brief = i18n("Kernel-based Virtual Machine Access");
         description = i18n("Allows running other operating systems as guests in virtual machines");
-        m_permissions.append(FlatpakPermission(brief, description, u"virt-manager-symbolic"_s));
+        permissions.append(FlatpakPermission(brief, description, u"virt-manager-symbolic"_s));
     }
 
     QStringList homeList, systemList;
@@ -958,7 +1006,7 @@ void FlatpakResource::loadPermissions()
         } else {
             description = i18n("Can access files in the following locations in your home folder without asking permission first: %1", appendText);
         }
-        m_permissions.append(FlatpakPermission(brief, description, u"user-home-symbolic"_s));
+        permissions.append(FlatpakPermission(brief, description, u"user-home-symbolic"_s));
     }
     appendText = createHtmlList(systemList);
     if (systemAccess) {
@@ -972,7 +1020,7 @@ void FlatpakResource::loadPermissions()
         } else {
             description = i18n("Can access system files in the following locations without asking permission first: %1", appendText);
         }
-        m_permissions.append(FlatpakPermission(brief, description, u"drive-harddisk-root-symbolic"_s));
+        permissions.append(FlatpakPermission(brief, description, u"drive-harddisk-root-symbolic"_s));
     }
 
     if (!fullSessionBusAccess) {
@@ -983,7 +1031,7 @@ void FlatpakResource::loadPermissions()
             description =
                 i18n("Can communicate with other applications and processes in the same desktop session using the following communication protocols: %1",
                      createHtmlList(busList));
-            m_permissions.append(FlatpakPermission(brief, description, "plugins-symbolic"_L1));
+            permissions.append(FlatpakPermission(brief, description, "plugins-symbolic"_L1));
         }
     }
 
@@ -994,9 +1042,10 @@ void FlatpakResource::loadPermissions()
             brief = i18n("System Bus Access");
             description =
                 i18n("Can communicate with all applications and system services using the following communication protocols: %1", createHtmlList(busList));
-            m_permissions.append(FlatpakPermission(brief, description, "plugins-symbolic"_L1));
+            permissions.append(FlatpakPermission(brief, description, "plugins-symbolic"_L1));
         }
     }
+    return permissions;
 }
 
 QString FlatpakResource::dataLocation() const
@@ -1029,6 +1078,12 @@ void FlatpakResource::clearUserData()
 
 int FlatpakResource::versionCompare(FlatpakResource *resource) const
 {
+    // This is getting called from QML, we better sanitize the input
+    if (!resource) {
+        qCWarning(LIBDISCOVER_BACKEND_FLATPAK_LOG) << "Cannot compare with a null resource";
+        return -1;
+    }
+
     const QString other = resource->availableVersion();
     return AppStream::Utils::vercmpSimple(availableVersion(), other);
 }
@@ -1038,16 +1093,6 @@ QString FlatpakResource::contentRatingDescription() const
     return AppStreamUtils::contentRatingDescription(m_appdata);
 }
 
-QString FlatpakResource::contentRatingText() const
-{
-    return AppStreamUtils::contentRatingText(m_appdata);
-}
-
-AbstractResource::ContentIntensity FlatpakResource::contentRatingIntensity() const
-{
-    return AppStreamUtils::contentRatingIntensity(m_appdata);
-}
-
 uint FlatpakResource::contentRatingMinimumAge() const
 {
     return AppStreamUtils::contentRatingMinimumAge(m_appdata);
@@ -1060,7 +1105,11 @@ QStringList FlatpakResource::topObjects() const
 
 QStringList FlatpakResource::bottomObjects() const
 {
-    return s_bottomObjects;
+    // Don't need permissions for non-apps; it doesn't make any sense
+    if (isDesktopApp()) {
+        return s_bottomObjects;
+    }
+    return {};
 }
 
 void FlatpakResource::addRefToUpdate(const QByteArray &toUpdate)
@@ -1073,4 +1122,22 @@ void FlatpakResource::clearToUpdate()
     m_toUpdate.clear();
 }
 
+bool FlatpakResource::hasResolvedIcon() const
+{
+    return m_icon.has_value();
+}
+
+QString FlatpakResource::verifiedMessage() const
+{
+    const bool flathubVerified = m_appdata.customValue(QStringLiteral("flathub::verification::verified")) == QLatin1String("true");
+    if (flathubVerified) {
+        bool ok;
+        const qint64 time = m_appdata.customValue(QStringLiteral("flathub::verification::timestamp")).toUInt(&ok);
+        return i18n("App Verified on %1 as shipped by its author %2",
+                    KFormat().formatRelativeDateTime(QDateTime::fromSecsSinceEpoch(time), QLocale::ShortFormat),
+                    author());
+    }
+    return {};
+}
+
 #include "moc_FlatpakResource.cpp"
